/*
 * Copyright (c) 2001, 2002, 2003, 2004, 2005
 * The Regents of The University of Michigan
 * All Rights Reserved
 *
 * This code is part of the M5 simulator.
 *
 * Permission is granted to use, copy, create derivative works and
 * redistribute this software and such derivative works for any
 * purpose, so long as the copyright notice above, this grant of
 * permission, and the disclaimer below appear in all copies made; and
 * so long as the name of The University of Michigan is not used in
 * any advertising or publicity pertaining to the use or distribution
 * of this software without specific, written prior authorization.
 *
 * THIS SOFTWARE IS PROVIDED AS IS, WITHOUT REPRESENTATION FROM THE
 * UNIVERSITY OF MICHIGAN AS TO ITS FITNESS FOR ANY PURPOSE, AND
 * WITHOUT WARRANTY BY THE UNIVERSITY OF MICHIGAN OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE. THE REGENTS OF THE UNIVERSITY OF MICHIGAN SHALL NOT BE
 * LIABLE FOR ANY DAMAGES, INCLUDING DIRECT, SPECIAL, INDIRECT,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, WITH RESPECT TO ANY CLAIM
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE SOFTWARE, EVEN
 * IF IT HAS BEEN OR IS HEREAFTER ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGES.
 *
 * Authors: Gabe Black
 */
#include <cstring>

#include "arch/mips/tlb.hh"

namespace MipsISA {
    Fault
    TLB::translate(RequestPtr req, ThreadContext *tc, bool)
    {
        Fault fault = GenericTLB::translate(req, tc);
        if (fault != NoFault)
            return fault;

        typeof(req->getSize()) size = req->getSize();
        Addr paddr = req->getPaddr();

        if (!isPowerOf2(size))
            panic("Invalid request size!\n");
        if ((size - 1) & paddr)
            return new GenericAlignmentFault(paddr);

        return NoFault;
    }

    void
    TlbEntry::serialize(std::ostream &os)
    {
        SERIALIZE_SCALAR(_pageStart);
    }

    void
    TlbEntry::unserialize(Checkpoint *cp, const std::string &section)
    {
        UNSERIALIZE_SCALAR(_pageStart);
    }
};

MipsISA::ITB *
MipsITBParams::create()
{
    return new MipsISA::ITB(this);
}

MipsISA::DTB *
MipsDTBParams::create()
{
    return new MipsISA::DTB(this);
}
